%% FM Demodulator
% Bestimmung von (SNR)NF in Abhngigkeit von (SNR)HF
% C. Roppel, FH Schmalkalden
% 11.08.2010
% 
% Erluterungen siehe "Analoge Modulationsverfahren und Rundfunktechnik",
% Begleitmaterial zum Buch "Grundlagen der digitalen Kommunikationstechnik"
% Fachbuchverlag Leipzig, 2006
%
% Verwendet die Funktion FM_IQdemod.m
% Getestet mit Matlab 7.4.0 und 7.9.0
%
clear
clc
close all
format short

%% Definitionen
fc = 2048000;        % Trgerfrequenz in Hz
fdelta = 75000;      % Frequenzhub in Hz
fg_nf = 60000;       % NF-Bandbreite in Hz
fsc = 8e6;           % Abtastrate in Hz
nsamples = 2e5;      % Anzahl Samples

snr_hf_list = [-12 -10 -8 -6 -4 -2 0 5 10 15];   % Liste (SNR)HF

beta = fdelta/(fg_nf)       % FM Modulationsindex
B_HF = 2*(beta + 1)*fg_nf   % HF-Bandbreite

% HF Bandpassfilter
fir_hf = fir1(256, [(fc - B_HF/2)/(fsc/2) (fc + B_HF/2)/(fsc/2)]);
% NF Tiefpassfilter
fir_lp = fir1(256, fg_nf/(fsc/2));

%% Erzeugung des FM Testsignals
% NF-Signal, Normierung der Amplitude auf 1
t = (0:(1/fsc):((nsamples - 1)/fsc));
xsin = sin(2*pi*1300*t) + sin(2*pi*57000*t);
xsin = xsin/max(abs(xsin));
Sx = mean(xsin.^2)   % Leistung des NF-Signals
% FM-Modulation, verwendet Matlab-Funktion fmmod()
xc = fmmod(xsin, fc, fsc, fdelta);
figure; periodogram(xc, [], [], fsc)
Sxc = mean(xc.^2)    % Leistung des HF-Signals

%% Rauschfreies HF-Referenzsignal
xrref = filter(fir_hf, 1, xc);     % HF Bandpassfilter
% Entferne die ersten Samples (Einschwingvorgang des FIR-Filters)
xrref = xrref(500:end);

%% Rauschfreies NF-Referenzsignal
yref = FM_IQdemod(xrref, fc, fsc, B_HF);
yref = filter(fir_lp, 1, yref);    % NF Tiefpassfilter
Syref = mean(yref.^2)              % Leistung des NF-Signals

%% Schleife fr verschiedene (SNR)HF
snr_hf_est_list = zeros(1, length(snr_hf_list));    % aus Simulation bestimmte Werte (SNR)HF
snr_nf_list = zeros(1, length(snr_hf_list));        % theoretische Werte (SNR)NF
snr_nf_est_list = zeros(1, length(snr_hf_list));    % aus Simulation bestimmte Werte (SNR)NF

for k = 1:length(snr_hf_list)
    snr_hf = snr_hf_list(k);
    xr = awgn(xc, snr_hf, 'measured', 12345);  % AWGN channel
    xr = filter(fir_hf, 1, xr);
    xr = xr(500:end);
    nhf = xr - xrref;                  % HF-Rauschsignal
    Nhf = mean(nhf.^2);                % HF-Rauschleistung
    snr_hf_est = 10*log10(Sxc/Nhf);    % (SNR)HF
    snr_hf_est_list(k) = snr_hf_est;
    snr_nf_list(k) = 10*log10(6*beta^2*(beta + 1)*Sx) + snr_hf_est;
    
    y = FM_IQdemod(xr, fc, fsc, B_HF); % Demodulation
    y = filter(fir_lp, 1, y);
    r = y - yref;                      % NF-Rauschsignal
    Nr = mean(r.^2);                   % NF-Rauschleistung
    snr_nf_est = 10*log10(Syref/Nr);   % (SNR)NF
    snr_nf_est_list(k) = snr_nf_est;
    fprintf('k: %d  (SNR)HF: %f  (SNR)NF: %f\n', k, snr_hf_est, snr_nf_est);
end

%% Plots
figure; plot(snr_hf_est_list, snr_nf_list, '--r', snr_hf_est_list, snr_nf_est_list, '-bo')
grid; xlabel('(S/N)_H_F in dB'); ylabel('(S/N)_N_F in dB');
legend('Theoretisch', 'FM Quadraturdemodulator', 'Location', 'SouthEast')
