%% Simulation einer QPSK-bertragungsstrecke mit einem Matched-Filter-Empfnger 
% C. Roppel, FH Schmalkalden
% 31.08.2010
% 
% Erluterungen siehe "Versuche mit Matlab",
% Begleitmaterial zum Buch "Grundlagen der digitalen Kommunikationstechnik"
% Fachbuchverlag Leipzig, 2006
%
% Getestet mit Matlab 7.4.0 und 7.9.0
%
clear
clc
close all
format short

%% Modulationsparameter fr QPSK
m = 4;          % m-wertige Symbole
k = log2(m);    % k Bit pro Symbol
alpha = 0.5;    % Roll-off-Faktor
nsym = 5e5;     % Anzahl der Symbole
os = 8;         % Samples pro Symbol (oversampling ratio)
dly_rc = 3;     % Verzgerung des Wurzel-Kosinus-roll-off-Filters

% Modulator und Demodulator
modobj = modem.pskmod('M', m, 'PhaseOffset', pi/4, 'SymbolOrder', 'Gray')
demodobj = modem.pskdemod('M', m, 'PhaseOffset', pi/4, 'SymbolOrder', 'Gray')

%% Bit- und Symbolfolge
dbit = randint(nsym*k, 1, [0 1], 12345);
dsym = bi2de(reshape(dbit, k, nsym).', 'left-msb');

%% Kanalmodell
% mehrwege = 0: verzerrungsfreier Kanal
% mehrwege = 1: Mehrwegekanal
mehrwege = 0;
dt = 2;         % Laufzeitdifferenz (in Samples)
a = 0.8;        % Dmpfung

% Impulsantwort des Kanals
if mehrwege == 0
    hch = 1; 
else
    hch = zeros(1, dt+1);
    hch(1) = 1; 
    hch(dt+1) = a;
end

%% Modulation - Kanal - Empfnger
% Modulation
xmod = modulate(modobj, dsym);

% Pulsformfilter (Wurzel-Kosinus-roll-off-Filter)
x = rcosflt(xmod, 1, os, 'fir/sqrt', alpha, dly_rc);
x = x((os*dly_rc + 1):(end - os*dly_rc));
eyediagram(x(os*dly_rc:10000), 2*os, 2, 1);
figure; periodogram(real(x), [], 4096, os);

% Faltung mit der Kanalimpulsantwort
y = filter(hch, 1, x);
figure; periodogram(real(y), [], 4096, os);

% Augendiagramm nach dem signalangepassten Filter, kein Rauschen
ymf = rcosflt(y, 1, os, 'fir/sqrt/Fs', alpha, dly_rc);
ymf = ymf((os*dly_rc + 1):(end - os*dly_rc));
eyediagram(ymf(os*dly_rc:10000), 2*os, 2, 1);

% Schleife fr verschiedene Eb/n0
Ebn0List = [6:10];                        % Eb/n0 in dB
berTheoList = zeros(1, length(Ebn0List)); % theoretsche Werte
berList = zeros(1, length(Ebn0List));     % aus Simulation bestimmte Werte

for i = 1:length(Ebn0List)    
    % AWGN-Kanal
    Ebn0 = Ebn0List(i);
    snr = Ebn0 + 10*log10(k) - 10*log10(os);
    r = awgn(y, snr, 'measured', 12345);
    % Empfnger: signalangepasstes Filter, Demodulation
    rmf = rcosflt(r, 1, os, 'fir/sqrt/Fs', alpha, dly_rc);
    rmf = rmf((os*dly_rc + 1):(end - os*dly_rc));
    rmf = downsample(rmf, os);    
    ysym = demodulate(demodobj, rmf);
    % Bestimmung der Bitfehlerhufigkeit
    ybit = de2bi(ysym, 'left-msb');
    ybit = reshape(ybit.', prod(size(ybit)), 1);
    [nerror, ber]= biterr(dbit, ybit);
    berList(i) = ber;   
    % Theoretische Werte 
    berTheoList(i) = 0.5*erfc(sqrt(10^(Ebn0/10)));
    fprintf('i: %d  Bitfehler: %d  Bitfehlerhufigkeit: %f\n', i, nerror, ber);
end
scatterplot(rmf(1:1e4));

%% Plot BER vs. Eb/n0
figure; semilogy(Ebn0List, berList, '-.or'); grid; title('QPSK Matched Filter');
xlabel('E_b/n_0 in dB'); ylabel('Bitfehlerhufigkeit'); hold on;
plot(Ebn0List, berTheoList,'-x'); hold off;
legend('Simulation', 'Theoretisch', 'Location','SouthWest');
